//#define DEBUG 1
#include <bios.h>
#include <conio.h>
#include <dos.h>
#include <graphics.h>
#include <math.h>
#include <stdio.h>
#define ESC_KEY 283
#define I_KEY 5993
#define P_KEY 6512
#define C_KEY 11875
#define IN 0
#define OUT 1
#define SQR2 1.4142
#define SQR3 1.7321

float XSCALE = 200, YSCALE = 200;
float XCENTER = 320, YCENTER = 240;

class vector{
   public:
   float x, y, z, vx, vy, vz;
   vector(){
       x = 0;
       y = 0;
       z = 0;
       vx = 0;
       vy = 0;
       vz = 0;
   }
   vector(float ax, float ay, float az, float avx, float avy, float avz){
       x = ax;
       y = ay;
       z = az;
       vx = avx;
       vy = avy;
       vz = avz;
   }
   void out(){
       printf("x=%f\t", x);
       printf("y=%f\t", y);
       printf("y=%f\t", z);
       printf("vx=%f\t", vx);
       printf("vy=%f\t", vy);
       printf("y=%f\t", vz);
   }
};
vector operator +(vector a, vector b){
   vector c;
   c.x = a.x + b.x;
   c.y = a.y + b.y;
   c.z = a.z + b.z;
   c.vx = a.vx + b.vx;
   c.vy = a.vy + b.vy;
   c.vz = a.vz + b.vz;
   return(c);
}
vector operator -(vector a, vector b){
   vector c;
   c.x = a.x - b.x;
   c.y = a.y - b.y;
   c.z = a.z - b.z;
   c.vx = a.vx - b.vx;
   c.vy = a.vy - b.vy;
   c.vz = a.vz - b.vz;
   return(c);
}
vector operator *(float a, vector b){
   vector c;
   c.x = a * b.x;
   c.y = a * b.y;
   c.z = a * b.z;
   c.vx = a * b.vx;
   c.vy = a * b.vy;
   c.vz = a * b.vz;
   return(c);
}
vector operator *(vector b, float a){
   vector c;
   c.x = a * b.x;
   c.y = a * b.y;
   c.z = a * b.z;
   c.vx = a * b.vx;
   c.vy = a * b.vy;
   c.vz = a * b.vz;
   return(c);
}
vector operator *(vector a, vector b){
   vector c;
   c.x = a.y * b.z - a.z * b.y;
   c.y = a.z * b.x - a.x * b.z;
   c.z = a.x * b.y - a.y * b.x;

   return(c);
}
vector FieldB(vector r){
   vector res;
   float m = sqrt(r.x*r.x + r.y*r.y + r.z*r.z);
   res.x = (-3 * r.x * r.z )/(m*m*m*m*m);
   res.y = (-3 * r.y * r.z )/(m*m*m*m*m);
   res.z = (-3 * r.z * r.z + m * m )/(m*m*m*m*m);
   return(res);
}
vector f(vector u, float t){
   vector res;
   vector B;
   vector E;
   vector v;
   vector vB;

   res.x = u.vx;
   res.y = u.vy;
   res.z = u.vz;

   B = FieldB(u);

   v = vector(u.vx, u.vy, u.vz, 0, 0, 0);
   vB = v*B;

   res.vx = vB.x;
   res.vy = vB.y;
   res.vz = vB.z;

   return(res);
}
void main(){
   vector k1, k2, k3, k4;
   float dt = 0.1, t=0;
   vector u(0, 1, 0, 0.01, 0, 0.01);
#ifndef DEBUG
   int gd=VGA, gm=VGAHI;
   initgraph(&gd, &gm, "egavga.bgi");
#endif
   while(1){
       if(bioskey(1)!=0){
	   int key = bioskey(0);
	   if(key==ESC_KEY) break;
           if(key==C_KEY){
               cleardevice();
           }
	   if(key==I_KEY){
	       u.vx = -u.vx;
	       u.vy = -u.vy;
               u.vz = -u.vz;
	   }
       }
       float sx, sy;

       sx = u.y * SQR3/2 - u.x * SQR3/2;
       sy = u.z - u.y/2 - u.x/2;
#ifndef DEBUG
       putpixel(XCENTER + sx*XSCALE, YCENTER - sy*YSCALE, WHITE);
#endif

       k1 = f(u, t)*dt;
       k2 = f(u + 0.5*k1, t+dt/2)*dt;
       k3 = f(u + 0.5*k2, t+dt/2)*dt;
       k4 = f(u + k3, t+dt)*dt;
       u = u + 1.0/6.0 * (k1 + 2*k2 + 2*k3 + k4);
       t += dt;
   }
#ifndef DEBUG
   setcolor(WHITE);
   outtextxy(10,10,"Press any key to continue...");
#endif
   getch();
   closegraph();
}